#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import (unicode_literals, division, absolute_import,
                        print_function)

__license__   = 'GPL v3'
__copyright__ = '2024 seeder'
__docformat__ = 'restructuredtext en'

import copy
from calibre_plugins.pitaval.shared.prefs import PluginPrefsName, MetadataIdentifier, MetadataName
from calibre.utils.config import JSONConfig

try:
    load_translations()
except NameError:
    pass # load_translations() added in calibre 1.9

LINE_OPTIONS = [
        (MetadataIdentifier.CUSTOM_TEXT, True, MetadataName.CUSTOM_TEXT),
        (MetadataIdentifier.TITLE, True, MetadataName.TITLE),
        (MetadataIdentifier.ORIGINAL_TITLE, True, MetadataName.ORIGINAL_TITLE),
        (MetadataIdentifier.PUBLISHER, True, MetadataName.PUBLISHER),
        (MetadataIdentifier.AUTHORS, True, MetadataName.AUTHORS),
        (MetadataIdentifier.TRANSLATION, True, MetadataName.TRANSLATION),
        (MetadataIdentifier.ILLUSTRATION, True, MetadataName.ILLUSTRATION),
        (MetadataIdentifier.COVER_AUTHORS, True, MetadataName.COVER_AUTHORS),
        (MetadataIdentifier.ORIGINAL_YEAR, True, MetadataName.ORIGINAL_YEAR),
        (MetadataIdentifier.PUB_YEAR, True, MetadataName.PUB_YEAR),
        (MetadataIdentifier.ORIGINAL_DATE, True, MetadataName.ORIGINAL_DATE),
        (MetadataIdentifier.PUB_DATE, True, MetadataName.PUB_DATE),
        (MetadataIdentifier.PAGES, True, MetadataName.PAGES),
        (MetadataIdentifier.COVER_TYPE, True, MetadataName.COVER_TYPE),
        (MetadataIdentifier.PRINT_RUN, True, MetadataName.PRINT_RUN),
        (MetadataIdentifier.SERIES, True, MetadataName.SERIES),
        (MetadataIdentifier.SERIES_INDEX, True, MetadataName.SERIES_INDEX),
        (MetadataIdentifier.EDITION, True,MetadataName.EDITION),
        (MetadataIdentifier.EDITION_INDEX, True, MetadataName.EDITION_INDEX),
        (MetadataIdentifier.CATEGORY, True, MetadataName.CATEGORY),
        (MetadataIdentifier.RATING, True, MetadataName.RATING),
        (MetadataIdentifier.RATING10, True, MetadataName.RATING10),
        (MetadataIdentifier.RATING5, True, MetadataName.RATING5),
        (MetadataIdentifier.RATING_COUNT, True, MetadataName.RATING_COUNT),
        (MetadataIdentifier.ISBN, True, MetadataName.ISBN),
        (MetadataIdentifier.ORIG_ID, True, MetadataName.ORIG_ID),
        (MetadataIdentifier.SOURCE_RELEVANCE, True, MetadataName.SOURCE_RELEVANCE),
        (MetadataIdentifier.EAN, True, MetadataName.EAN),
        (MetadataIdentifier.PRICE, True, MetadataName.PRICE),
        (MetadataIdentifier.LANGUAGE, True, MetadataName.LANGUAGE),
        (MetadataIdentifier.DIMENSIONS, True, MetadataName.DIMENSIONS),
        (MetadataIdentifier.ISSUE_NUMBER, True, MetadataName.ISSUE_NUMBER),
        (MetadataIdentifier.ALT_TITLE, True, MetadataName.ALT_TITLE),
        (MetadataIdentifier.SEP_PUBLISHED, True, MetadataName.SEP_PUBLISHED),
        (MetadataIdentifier.WORLD, True, MetadataName.WORLD),
        (MetadataIdentifier.ALT_ORIGINAL_TITLE, True, MetadataName.ALT_ORIGINAL_TITLE),
        (MetadataIdentifier.SUBTITLE, True, MetadataName.SUBTITLE),
        (MetadataIdentifier.NOTE, True, MetadataName.NOTE)
        ]

TAG_OPTIONS = [
            (MetadataIdentifier.CATEGORY, True, MetadataName.CATEGORY),
            (MetadataIdentifier.PAGES, False, MetadataName.PAGES),
            (MetadataIdentifier.COVER_TYPE, False, MetadataName.COVER_TYPE),
            (MetadataIdentifier.PRINT_RUN, False, MetadataName.PRINT_RUN),
            (MetadataIdentifier.ORIGINAL_TITLE, False, MetadataName.ORIGINAL_TITLE),
            (MetadataIdentifier.TITLE, False, MetadataName.TITLE),
            (MetadataIdentifier.ORIGINAL_YEAR, False, MetadataName.ORIGINAL_YEAR),
            (MetadataIdentifier.PUB_YEAR, False, MetadataName.PUB_YEAR),
            (MetadataIdentifier.ORIGINAL_DATE, False, MetadataName.ORIGINAL_DATE),
            (MetadataIdentifier.PUB_DATE, False, MetadataName.PUB_DATE),
            (MetadataIdentifier.PUBLISHER, False, MetadataName.PUBLISHER),
            (MetadataIdentifier.AUTHORS, False, MetadataName.AUTHORS),
            (MetadataIdentifier.TRANSLATION, False, MetadataName.TRANSLATION),
            (MetadataIdentifier.ILLUSTRATION, False, MetadataName.ILLUSTRATION),
            (MetadataIdentifier.COVER_AUTHORS, False, MetadataName.COVER_AUTHORS),
            (MetadataIdentifier.SERIES, False, MetadataName.SERIES),
            (MetadataIdentifier.EDITION, False, MetadataName.EDITION),
            (MetadataIdentifier.RATING, False, MetadataName.RATING),
            (MetadataIdentifier.RATING10, False, MetadataName.RATING10),
            (MetadataIdentifier.RATING5, False, MetadataName.RATING5),
            (MetadataIdentifier.RATING_COUNT, False, MetadataName.RATING_COUNT),
            (MetadataIdentifier.ISBN, False, MetadataName.ISBN),
            (MetadataIdentifier.ORIG_ID, False, MetadataName.ORIG_ID),
            (MetadataIdentifier.AWARDS, False, MetadataName.AWARDS),
            (MetadataIdentifier.TALES_IN_BOOK, False, MetadataName.TALES_IN_BOOK),
            (MetadataIdentifier.CONTAINED_IN, False, MetadataName.CONTAINED_IN),
            (MetadataIdentifier.SOURCE_RELEVANCE, False, MetadataName.SOURCE_RELEVANCE),
            (MetadataIdentifier.EAN, False, MetadataName.EAN),
            (MetadataIdentifier.PRICE, False, MetadataName.PRICE),
            (MetadataIdentifier.LANGUAGE, False, MetadataName.LANGUAGE),
            (MetadataIdentifier.DIMENSIONS, False, MetadataName.DIMENSIONS),
            (MetadataIdentifier.ISSUE_NUMBER, False, MetadataName.ISSUE_NUMBER),
            (MetadataIdentifier.ALT_TITLE, False, MetadataName.ALT_TITLE),
            (MetadataIdentifier.SEP_PUBLISHED, False, MetadataName.SEP_PUBLISHED),
            (MetadataIdentifier.WORLD, False, MetadataName.WORLD),
            (MetadataIdentifier.ALT_ORIGINAL_TITLE, False, MetadataName.ALT_ORIGINAL_TITLE),
            (MetadataIdentifier.SUBTITLE, False, MetadataName.SUBTITLE),
            (MetadataIdentifier.NOTE, False, MetadataName.NOTE)
        ]

IDENTIFIER_OPTIONS = [
            (MetadataIdentifier.ORIG_ID, True, MetadataName.ORIG_ID, 'pitaval'),
            (MetadataIdentifier.ISBN, True, MetadataName.ISBN, 'isbn'),
            (MetadataIdentifier.EAN, False, MetadataName.EAN, 'ean'),
            (MetadataIdentifier.RATING, False, MetadataName.RATING, 'pit_rating100'),
            (MetadataIdentifier.RATING10, False, MetadataName.RATING10, 'pit_rating10'),
            (MetadataIdentifier.RATING5, False, MetadataName.RATING5, 'pit_rating5'),
            (MetadataIdentifier.RATING_COUNT, False, MetadataName.RATING_COUNT, 'pit_ratingcount'),
            (MetadataIdentifier.PAGES, False, MetadataName.PAGES, 'pit_pages'),
            (MetadataIdentifier.PRINT_RUN, False, MetadataName.PRINT_RUN, 'pit_printrun'),
            (MetadataIdentifier.ORIGINAL_YEAR, False, MetadataName.ORIGINAL_YEAR, 'pit_origyear'),
            (MetadataIdentifier.ORIGINAL_DATE, False, MetadataName.ORIGINAL_DATE, 'pit_origdate'),
            (MetadataIdentifier.PUB_YEAR, False, MetadataName.PUB_YEAR, 'pit_pubyear'),
            (MetadataIdentifier.PUB_DATE, False, MetadataName.PUB_DATE, 'pit_pubdate'),
            (MetadataIdentifier.COVER_TYPE, False, MetadataName.COVER_TYPE, 'pit_covertype'),
            (MetadataIdentifier.ORIGINAL_TITLE, False, MetadataName.ORIGINAL_TITLE, 'pit_origtitle'),
            (MetadataIdentifier.TITLE, False, MetadataName.TITLE, 'pit_title'),
            (MetadataIdentifier.PUBLISHER, False, MetadataName.PUBLISHER, 'pit_publisher'),
            (MetadataIdentifier.AUTHORS, False, MetadataName.AUTHORS, 'pit_authors'),
            (MetadataIdentifier.TRANSLATION, False, MetadataName.TRANSLATION, 'pit_translators'),
            (MetadataIdentifier.ILLUSTRATION, False, MetadataName.ILLUSTRATION, 'pit_illustrators'),
            (MetadataIdentifier.COVER_AUTHORS, False, MetadataName.COVER_AUTHORS, 'pit_coverauthors'),
            (MetadataIdentifier.SERIES, False, MetadataName.SERIES, 'pit_series'),
            (MetadataIdentifier.EDITION, False, MetadataName.EDITION, 'pit_edition'),
            (MetadataIdentifier.RATING_EMOTIONS, False, MetadataName.RATING_EMOTIONS, 'pit_emotions'),
            (MetadataIdentifier.AWARDS, False, MetadataName.AWARDS, 'pit_awards'),
            (MetadataIdentifier.TALES_IN_BOOK, False, MetadataName.TALES_IN_BOOK, 'pit_bookparts'),
            (MetadataIdentifier.CONTAINED_IN, False, MetadataName.CONTAINED_IN, 'pit_containedin'),
            (MetadataIdentifier.SOURCE_RELEVANCE, False, MetadataName.SOURCE_RELEVANCE, 'pit_relevance'),
            (MetadataIdentifier.PRICE, False, MetadataName.PRICE, 'pit_price'),
            (MetadataIdentifier.LANGUAGE, False, MetadataName.LANGUAGE, 'pit_language'),
            (MetadataIdentifier.DIMENSIONS, False, MetadataName.DIMENSIONS, 'pit_dimensions'),
            (MetadataIdentifier.ISSUE_NUMBER, False, MetadataName.ISSUE_NUMBER, 'pit_issuenumber'),
            (MetadataIdentifier.ALT_TITLE, False, MetadataName.ALT_TITLE, 'pit_alttitle'),
            (MetadataIdentifier.SEP_PUBLISHED, False, MetadataName.SEP_PUBLISHED, 'pit_seppublished'),
            (MetadataIdentifier.WORLD, False, MetadataName.WORLD, 'pit_world'),
            (MetadataIdentifier.ALT_ORIGINAL_TITLE, False, MetadataName.ALT_ORIGINAL_TITLE, 'pit_altoriginaltitle'),
            (MetadataIdentifier.SUBTITLE, False, MetadataName.SUBTITLE, 'pit_subtitle'),
            (MetadataIdentifier.NOTE, False, MetadataName.NOTE, 'pit_note'),
            (MetadataIdentifier.CATEGORY, False, MetadataName.CATEGORY, 'pit_category'),
        ]

COMMENT_OPTIONS = [
        (MetadataIdentifier.DESCRIPTION, True, MetadataName.DESCRIPTION),
        (MetadataIdentifier.HR, True, MetadataName.HR),
        (MetadataIdentifier.PAGES, True, MetadataName.PAGES),
        (MetadataIdentifier.COVER_TYPE, True, MetadataName.COVER_TYPE),
        (MetadataIdentifier.PRINT_RUN, True, MetadataName.PRINT_RUN),
        (MetadataIdentifier.ORIGINAL_TITLE, True, MetadataName.ORIGINAL_TITLE),
        (MetadataIdentifier.TITLE, True, MetadataName.TITLE),
        (MetadataIdentifier.ORIGINAL_YEAR, True, MetadataName.ORIGINAL_YEAR),
        (MetadataIdentifier.PUB_YEAR, True, MetadataName.PUB_YEAR),
        (MetadataIdentifier.ORIGINAL_DATE, True, MetadataName.ORIGINAL_DATE),
        (MetadataIdentifier.PUB_DATE, True, MetadataName.PUB_DATE),
        (MetadataIdentifier.PUBLISHER, True, MetadataName.PUBLISHER),
        (MetadataIdentifier.AUTHORS, True, MetadataName.AUTHORS),
        (MetadataIdentifier.TRANSLATION, True, MetadataName.TRANSLATION),
        (MetadataIdentifier.ILLUSTRATION, True, MetadataName.ILLUSTRATION),
        (MetadataIdentifier.COVER_AUTHORS, True, MetadataName.COVER_AUTHORS),
        (MetadataIdentifier.SERIES, True, MetadataName.SERIES),
        (MetadataIdentifier.EDITION, True, MetadataName.EDITION),
        (MetadataIdentifier.GENRES, True, MetadataName.GENRES),
        (MetadataIdentifier.TAGS, True, MetadataName.TAGS),
        (MetadataIdentifier.RATING, True, MetadataName.RATING),
        (MetadataIdentifier.RATING10, True, MetadataName.RATING10),
        (MetadataIdentifier.RATING5, True, MetadataName.RATING5),
        (MetadataIdentifier.RATING_COUNT, True, MetadataName.RATING_COUNT),
        (MetadataIdentifier.ISBN, True, MetadataName.ISBN),
        (MetadataIdentifier.ORIG_ID, True, MetadataName.ORIG_ID),
        (MetadataIdentifier.AWARDS, True, MetadataName.AWARDS),
        (MetadataIdentifier.TRIVIA, True, MetadataName.TRIVIA),
        (MetadataIdentifier.TALES_IN_BOOK, True, MetadataName.TALES_IN_BOOK),
        (MetadataIdentifier.CONTAINED_IN, True, MetadataName.CONTAINED_IN),
        (MetadataIdentifier.SOURCE_RELEVANCE, True, MetadataName.SOURCE_RELEVANCE),
        (MetadataIdentifier.EAN, True, MetadataName.EAN),
        (MetadataIdentifier.PRICE, True, MetadataName.PRICE),
        (MetadataIdentifier.LANGUAGE, True, MetadataName.LANGUAGE),
        (MetadataIdentifier.DIMENSIONS, True, MetadataName.DIMENSIONS),
        (MetadataIdentifier.ISSUE_NUMBER, True, MetadataName.ISSUE_NUMBER),
        (MetadataIdentifier.ALT_TITLE, True, MetadataName.ALT_TITLE),
        (MetadataIdentifier.SEP_PUBLISHED, True, MetadataName.SEP_PUBLISHED),
        (MetadataIdentifier.WORLD, True, MetadataName.WORLD),
        (MetadataIdentifier.ALT_ORIGINAL_TITLE, True, MetadataName.ALT_ORIGINAL_TITLE),
        (MetadataIdentifier.SUBTITLE, True, MetadataName.SUBTITLE),
        (MetadataIdentifier.NOTE, True, MetadataName.NOTE),
        (MetadataIdentifier.CATEGORY, True, MetadataName.CATEGORY),
        (MetadataIdentifier.EXTERNAL_LINKS, True, MetadataName.EXTERNAL_LINKS),
        ]

DEFAULT_CATEGORY_MAPPINGS = {
        'akční': ['akční'],
        'anglická škola': ['anglická škola'],
        'antologie': ['antologie'],
        'detektivní': ['detektivní'],
        'drsná škola': ['drsná škola'],
        'fantaskní': ['fantaskní'],
        'gangsterka': ['gangsterka'],
        'historické': ['historické'],
        'horor': ['horor'],
        'humorná': ['humorná'],
        'konspirace': ['konspirace'],
        'krimi': ['krimi'],
        'mafiánská': ['mafiánská'],
        'omnibus': ['omnibus'],
        'povídková sbírka': ['povídková sbírka'],
        'právnické': ['právnické'],
        'psychologická': ['psychologická'],
        'sci-fi': ['sci-fi'],
        'thriller': ['thriller'],
        'vězeňská': ['vězeňská'],
        'záhada zamčeného pokoje': ['záhada zamčeného pokoje'],
        'špionážní': ['špionážní']
        }

DEFAULT_STORE_VALUES = {
    PluginPrefsName.KEY_MAX_DOWNLOADS: 10,
    PluginPrefsName.MAX_COVERS: 1,
    PluginPrefsName.OBALKYKNIH_COVER: False,
    PluginPrefsName.BOOKPORT_COVER: False,
    PluginPrefsName.IDENTIFIER_SEARCH: True,
    PluginPrefsName.ISBN_SEARCH: True,
    PluginPrefsName.TALES_SEARCH: False,
    PluginPrefsName.GOOGLE_SEARCH: False,
    PluginPrefsName.DUCKDUCKGO_SEARCH: False,
    PluginPrefsName.ISSUE_PREFERENCE: 0,
    PluginPrefsName.AUTHORS_INCLUDE: True,
    PluginPrefsName.TRANSLATORS_INCLUDE: False,
    PluginPrefsName.ILLUSTRATORS_INCLUDE: False,
    PluginPrefsName.COVER_AUTHORS_INCLUDE: False,
    PluginPrefsName.APPEND_TO_COMMENTS: [
        (MetadataIdentifier.DESCRIPTION, True, MetadataName.DESCRIPTION),
    ],
    PluginPrefsName.APPEND_TO_TAG: TAG_OPTIONS,
    PluginPrefsName.APPEND_TO_IDENTIFIERS: IDENTIFIER_OPTIONS,
    PluginPrefsName.SERIES_LOCALIZATION: 0,
    PluginPrefsName.PUBLICATION_DATE: 0,
    PluginPrefsName.SWAP_AUTHORS: False,
    PluginPrefsName.ONE_AUTHOR: False,
    PluginPrefsName.AUTHOR_ROLE: False,
    PluginPrefsName.CALCULATE_RATING: False,
    PluginPrefsName.CALCULATE_RATING_ACCURACY: 3,
    PluginPrefsName.CATEGORY_FILTER: False,
    PluginPrefsName.SERIES_FILTER: False,
    PluginPrefsName.PUBLISHER_FILTER: False,
    PluginPrefsName.PUBLISHER_LINE: [
        (MetadataIdentifier.PUBLISHER, True, MetadataName.PUBLISHER),
    ],
    PluginPrefsName.TITLE_LINE: [
        (MetadataIdentifier.TITLE, True, MetadataName.TITLE),
    ],
    PluginPrefsName.SERIES_LINE: [
        (MetadataIdentifier.SERIES, True, MetadataName.SERIES),
    ],
    PluginPrefsName.SERIES_INDEX_FIELD: (MetadataIdentifier.SERIES_INDEX, True, MetadataName.SERIES_INDEX),
    PluginPrefsName.SERIES_INDEXING_ITEM: 0,
    PluginPrefsName.KEY_SERIES_MAPPINGS: {},
    PluginPrefsName.KEY_CATEGORY_MAPPINGS: copy.deepcopy(DEFAULT_CATEGORY_MAPPINGS),
    PluginPrefsName.KEY_PUBLISHER_MAPPINGS: {}
}

# This is where all preferences for this plugin will be stored
plugin_prefs = JSONConfig('plugins/pitaval')
# Set defaults
plugin_prefs.defaults[PluginPrefsName.STORE_NAME] = DEFAULT_STORE_VALUES

def get_pref(option=None):
    c = plugin_prefs[PluginPrefsName.STORE_NAME]
    if option is not None:
        default_value = plugin_prefs.defaults[PluginPrefsName.STORE_NAME][option]
        return c.get(option, default_value)
    else:
        return plugin_prefs

def set_pref(new_prefs):
    plugin_prefs[PluginPrefsName.STORE_NAME] = new_prefs